@@ -39,6 +39,10 @@ extern void clear_wrapcontd(termline * line, int y);
 extern ushort getparabidi(termline * line);
 extern wchar * wcsline(termline * line);  // for debug output
 
+/* Advanced Arabic support */
+extern bool isLAM(xchar c);
+extern bool isALEF(xchar c);
+
 static inline bool
 term_selecting(void)
 { return term.mouse_state < 0 && term.mouse_state >= MS_SEL_LINE; }
@@ -64,7 +68,7 @@ extern termline * decompressline(uchar *, int * bytes_used);
 
 extern termchar * term_bidi_line(termline *, int scr_y);
 
-extern void term_export_html(bool do_open);
+extern void term_export_html(bool all, bool do_open);
 extern char * term_get_html(int level);
 extern void print_screen(void);
 
